@@ -26,6 +26,9 @@
 #include "calculate_util.h"
 #include "util.h"
 #include "appslistmodel.h"
+#include "fullscreenframe.h"
+
+#include <DGuiApplicationHelper>
 
 #include <QDebug>
 #include <QWheelEvent>
@@ -40,6 +43,8 @@
 #include <QScrollBar>
 #include <QVBoxLayout>
 
+DGUI_USE_NAMESPACE
+
 QPointer<AppsManager> AppGridView::m_appManager = nullptr;
 QPointer<CalculateUtil> AppGridView::m_calcUtil = nullptr;
 
@@ -48,12 +53,18 @@ Gesture *AppGridView::m_gestureInter = nullptr;
 
 /**
  * @brief AppGridView::AppGridView
- * 全屏模式下 单个应用分类的视图列表或者所有应用的视图列表
+ * 全屏模式下 应用网格视图，主要处理全屏图标的拖拽事件及分组切换的动画效果
  * @param parent
  */
 AppGridView::AppGridView(QWidget *parent)
     : QListView(parent)
     , m_dropThresholdTimer(new QTimer(this))
+    , m_pixLabel(nullptr)
+    , m_calendarWidget(nullptr)
+    , m_vlayout(nullptr)
+    , m_monthLabel(nullptr)
+    , m_dayLabel(nullptr)
+    , m_weekLabel(nullptr)
 {
     m_pDelegate = nullptr;
 
@@ -74,6 +85,7 @@ AppGridView::AppGridView(QWidget *parent)
 
     viewport()->installEventFilter(this);
     viewport()->setAcceptDrops(true);
+    createMovingComponent();
 
     setUniformItemSizes(true);
     setMouseTracking(true);
@@ -91,13 +103,9 @@ AppGridView::AppGridView(QWidget *parent)
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setFrameStyle(QFrame::NoFrame);
     setViewportMargins(0, 0, 0, 0);
-
-    // init origin size
-    setFixedSize(m_appManager->currentScreen()->size());
-
     viewport()->setAutoFillBackground(false);
 
-    // update item spacing
+    // 界面大小变化时，设置listview的边距以及间距
     connect(m_calcUtil, &CalculateUtil::layoutChanged, this, [this] {
         setSpacing(m_calcUtil->appItemSpacing());
         setViewportMargins(m_calcUtil->appMarginLeft(), m_calcUtil->appMarginTop(), m_calcUtil->appMarginLeft(), 0);
@@ -109,10 +117,6 @@ AppGridView::AppGridView(QWidget *parent)
     connect(m_dropThresholdTimer, &QTimer::timeout, this, &AppGridView::dropSwap);
 #endif
 
-    connect(m_appManager, &AppsManager::iconLoadFinished, this, [ = ] {
-        QListView::update();
-    });
-
     // 根据后端延迟触屏信号控制是否可进行图标拖动，收到延迟触屏信号可拖动，没有收到延迟触屏信号、点击松开就不可拖动
     connect(m_gestureInter, &Gesture::TouchSinglePressTimeout, m_gestureInter, [] {
         m_longPressed = true;
@@ -142,16 +146,6 @@ void AppGridView::setContainerBox(const QWidget *container)
     m_containerBox = container;
 }
 
-// FIXME(sbw):
-// some white/black lines occurred in non-integer scale hidpi.
-void AppGridView::updateItemHiDPIFixHook(const QModelIndex &index)
-{
-    const QRect &r = indexRect(index);
-    update(r.marginsRemoved(QMargins(0, 0, 1, 1)));
-//    update(r.marginsAdded(QMargins(-1, -1, -1, -1)));
-    //    QListView::update(index);
-}
-
 void AppGridView::setDelegate(DragPageDelegate *pDelegate)
 {
     m_pDelegate = pDelegate;
@@ -166,6 +160,13 @@ void AppGridView::dropEvent(QDropEvent *e)
 {
     e->accept();
 
+    if (m_dropThresholdTimer->isActive())
+        m_dropThresholdTimer->stop();
+
+    // 解决快速拖动应用，没有动画效果的问题，当拖拽小于200毫秒时，禁止拖动
+    if (m_dragLastTime.elapsed() < 200)
+        return;
+
     m_enableDropInside = true;
 
     // set the correct hover item.
@@ -192,13 +193,18 @@ void AppGridView::mousePressEvent(QMouseEvent *e)
                 }
             }
 
+            // 触发右键菜单
             emit popupMenuRequested(rightClickPoint, clickedIndex);
         }
     }
 
-    if (e->buttons() == Qt::LeftButton && !m_lastFakeAni)
+    if (e->buttons() == Qt::LeftButton && !m_lastFakeAni) {
         m_dragStartPos = e->pos();
 
+        // 记录动画的终点位置
+        setDropAndLastPos(appIconRect(indexAt(e->pos())).topLeft());
+    }
+
     if (m_pDelegate)
         m_pDelegate->mousePress(e);
 
@@ -207,6 +213,8 @@ void AppGridView::mousePressEvent(QMouseEvent *e)
 
 void AppGridView::dragEnterEvent(QDragEnterEvent *e)
 {
+    m_dragLastTime.start();
+
     const QModelIndex index = indexAt(e->pos());
 
     if (model()->canDropMimeData(e->mimeData(), e->dropAction(), index.row(), index.column(), QModelIndex())) {
@@ -244,8 +252,10 @@ void AppGridView::dragMoveEvent(QDragMoveEvent *e)
         }
     }
 
+    // 分页切换后隐藏label过渡效果
     emit requestScrollStop();
 
+    // 释放前执行app交换动画
     if (m_enableAnimation)
         m_dropThresholdTimer->start();
 }
@@ -259,9 +269,9 @@ void AppGridView::dragOut(int pos)
     dropSwap();
 }
 
-void AppGridView::dragIn(const QModelIndex &index)
+void AppGridView::dragIn(const QModelIndex &index, bool enableAnimation)
 {
-    m_enableAnimation = false;
+    m_enableAnimation = enableAnimation;
     m_dragStartPos = indexRect(index).center();
     AppsListModel *listModel = qobject_cast<AppsListModel *>(model());
     if (!listModel)
@@ -270,24 +280,36 @@ void AppGridView::dragIn(const QModelIndex &index)
     listModel->setDraggingIndex(index);
 }
 
+void AppGridView::setDropAndLastPos(const QPoint& itemPos)
+{
+    m_dropPoint = itemPos;
+}
+
 void AppGridView::flashDrag()
 {
     AppsListModel *listModel = qobject_cast<AppsListModel *>(model());
     int dragDropRow = listModel->dragDropIndex().row();
-    startDrag(indexAt(dragDropRow));
+
+    // 刷新的时候不执行drag，因为实际上并没有拖动的动作。在wayland环境中阻塞会在Drag::exec()函数中。
+    startDrag(indexAt(dragDropRow), false);
 }
 
+/**
+ * @brief AppGridView::dragLeaveEvent 离开listview时触发分页
+ * @param e 拖动离开事件指针对象
+ */
 void AppGridView::dragLeaveEvent(QDragLeaveEvent *e)
 {
     Q_ASSERT(m_containerBox);
 
     m_dropThresholdTimer->stop();
 
-    const QPoint pos = m_containerBox->mapFromGlobal(QCursor::pos());
-
-    int nSpace = m_calcUtil->appItemSpacing() + m_calcUtil->appMarginLeft();
-    const QRect containerRect = m_containerBox->rect().marginsRemoved(QMargins(nSpace, DLauncher::APP_DRAG_SCROLL_THRESHOLD,
-                                                                               nSpace, DLauncher::APP_DRAG_SCROLL_THRESHOLD));
+    // 获取光标在listview中的坐标
+    QPoint pos = mapFromGlobal(QCursor::pos());
+    int padding = m_calcUtil->appMarginLeft();
+    QMargins margin(padding, DLauncher::APP_DRAG_SCROLL_THRESHOLD,
+                    padding, DLauncher::APP_DRAG_SCROLL_THRESHOLD);
+    QRect containerRect = this->contentsRect().marginsRemoved(margin);
 
     const QModelIndex dropStart = QListView::indexAt(m_dragStartPos);
 
@@ -303,11 +325,11 @@ void AppGridView::mouseMoveEvent(QMouseEvent *e)
 {
     e->accept();
 
-    // disable qlistview default drag
     setState(NoState);
 
     const QModelIndex &idx = indexAt(e->pos());
 
+    // 鼠标在app上划过时实现选中效果
     if (idx.isValid())
         emit entered(idx);
 
@@ -319,16 +341,17 @@ void AppGridView::mouseMoveEvent(QMouseEvent *e)
         m_pDelegate->mouseMove(e);
 
     // 如果是触屏事件转换而来并且没有收到后端的延时触屏消息，不进行拖拽
-    if (e->source() == Qt::MouseEventSynthesizedByQt && !m_longPressed) {
+    if (e->source() == Qt::MouseEventSynthesizedByQt && !m_longPressed)
         return;
-    }
 
     if (qAbs(e->x() - m_dragStartPos.x()) > DLauncher::DRAG_THRESHOLD ||
             qAbs(e->y() - m_dragStartPos.y()) > DLauncher::DRAG_THRESHOLD) {
         m_moveGridView = true;
+
         // 开始拖拽后,导致fullscreenframe只收到mousePress事件,收不到mouseRelease事件,需要处理一下异常
         if (idx.isValid())
             emit requestMouseRelease();
+
         return startDrag(QListView::indexAt(m_dragStartPos));
     }
 }
@@ -345,10 +368,10 @@ void AppGridView::mouseReleaseEvent(QMouseEvent *e)
 
     int diff_x = qAbs(e->pos().x() - m_dragStartPos.x());
     int diff_y = qAbs(e->pos().y() - m_dragStartPos.y());
+
     // 小范围位置变化，当作没有变化，针对触摸屏
-    if (diff_x < DLauncher::TOUCH_DIFF_THRESH && diff_y < DLauncher::TOUCH_DIFF_THRESH) {
+    if (diff_x < DLauncher::TOUCH_DIFF_THRESH && diff_y < DLauncher::TOUCH_DIFF_THRESH)
         QListView::mouseReleaseEvent(e);
-    }
 }
 
 QPixmap AppGridView::creatSrcPix(const QModelIndex &index, const QString &appKey)
@@ -360,47 +383,49 @@ QPixmap AppGridView::creatSrcPix(const QModelIndex &index, const QString &appKey
         const double  iconZoom =  s.width() / 64.0;
         QStringList calIconList = m_calcUtil->calendarSelectIcon();
 
-        auto calendar = new QWidget() ;
-        calendar->setFixedSize(s);
-
-        calendar->setAutoFillBackground(true);
-        QPalette palette = calendar->palette();
+        m_calendarWidget->setFixedSize(s);
+        m_calendarWidget->setAutoFillBackground(true);
+        QPalette palette = m_calendarWidget->palette();
         palette.setBrush(QPalette::Window,
                          QBrush(QPixmap(calIconList.at(0)).scaled(
-                                    calendar->size(),
+                                    m_calendarWidget->size(),
                                     Qt::IgnoreAspectRatio,
                                     Qt::SmoothTransformation)));
-        calendar->setPalette(palette);
+        m_calendarWidget->setPalette(palette);
 
-        QVBoxLayout *layout = new QVBoxLayout;
-        layout->setSpacing(0);
-        auto month = new QLabel();
+        m_vlayout->setSpacing(0);
         auto monthPix = loadSvg(calIconList.at(1), QSize(20, 10) * iconZoom);
-        month->setPixmap(monthPix.scaled(monthPix.width(), monthPix.height(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
-        month->setFixedHeight(monthPix.height());
-        month->setAlignment(Qt::AlignCenter);
-        month->setFixedWidth(s.width() - 5 * iconZoom);
-        layout->addWidget(month, Qt::AlignVCenter);
+        m_monthLabel->setPixmap(monthPix.scaled(monthPix.width(), monthPix.height(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
+        m_monthLabel->setFixedHeight(monthPix.height());
+        m_monthLabel->setAlignment(Qt::AlignCenter);
+        m_monthLabel->setFixedWidth(s.width() - 5 * iconZoom);
+        m_monthLabel->show();
+        m_vlayout->addWidget(m_monthLabel, Qt::AlignVCenter);
 
-        auto day = new QLabel();
         auto dayPix = loadSvg(calIconList.at(2), QSize(28, 26) * iconZoom);
-        day->setPixmap(dayPix.scaled(dayPix.width(), dayPix.height(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
-        day->setAlignment(Qt::AlignCenter);
-        day->setFixedHeight(day->pixmap()->height());
-        day->raise();
-        layout->addWidget(day, Qt::AlignVCenter);
+        m_dayLabel->setPixmap(dayPix.scaled(dayPix.width(), dayPix.height(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
+        m_dayLabel->setAlignment(Qt::AlignCenter);
+        m_dayLabel->setFixedHeight(m_dayLabel->pixmap()->height());
+        m_dayLabel->raise();
+        m_dayLabel->show();
+        m_vlayout->addWidget(m_dayLabel, Qt::AlignVCenter);
 
-        auto week = new QLabel();
         auto weekPix = loadSvg(calIconList.at(3), QSize(14, 6) * iconZoom);
-        week->setPixmap(weekPix.scaled(weekPix.width(), weekPix.height(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
-        week->setFixedHeight(week->pixmap()->height());
-        week->setAlignment(Qt::AlignCenter);
-        week->setFixedWidth(s.width() + 5 * iconZoom);
-        layout->addWidget(week, Qt::AlignVCenter);
-        layout->setSpacing(0);
-        layout->setContentsMargins(0, 10 * iconZoom, 0, 10 * iconZoom);
-        calendar->setLayout(layout);
-        srcPix = calendar->grab(calendar->rect());
+        m_weekLabel->setPixmap(weekPix.scaled(weekPix.width(), weekPix.height(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
+        m_weekLabel->setFixedHeight(m_weekLabel->pixmap()->height());
+        m_weekLabel->setAlignment(Qt::AlignCenter);
+        m_weekLabel->setFixedWidth(s.width() + 5 * iconZoom);
+        m_weekLabel->show();
+
+        m_vlayout->addWidget(m_weekLabel, Qt::AlignVCenter);
+        m_vlayout->setSpacing(0);
+        m_vlayout->setContentsMargins(0, 10 * iconZoom, 0, 10 * iconZoom);
+        m_calendarWidget->setLayout(m_vlayout);
+        m_calendarWidget->show();
+
+        srcPix = m_calendarWidget->grab(m_calendarWidget->rect());
+        m_calendarWidget->hide();
+        m_calendarWidget->setLayout(nullptr);
     } else {
         srcPix = index.data(AppsListModel::AppDragIconRole).value<QPixmap>();
     }
@@ -408,10 +433,52 @@ QPixmap AppGridView::creatSrcPix(const QModelIndex &index, const QString &appKey
     return srcPix;
 }
 
-void AppGridView::startDrag(const QModelIndex &index)
+/**
+ * @brief AppGridView::appIconRect 计算应用图标的矩形位置，该接口中的浮点数据
+ * 是从视图代理paint接口中摘过来的matlab模拟的数据
+ * @param index 应用图标所在的模型索引
+ * @return 应用图标对应的矩形大小
+ */
+QRect AppGridView::appIconRect(const QModelIndex &index)
+{
+    qreal ratio = qApp->devicePixelRatio();
+    const QSize iconSize = index.data(AppsListModel::AppIconSizeRole).toSize() * ratio;
+
+    const static double x1 = 0.26418192;
+    const static double x2 = -0.38890932;
+
+    const int w = indexRect(index).width();
+    const int h = indexRect(index).height();
+    const int sub = qAbs((w - h) / 2);
+
+    QRect ibr;
+    if (w == h)
+         ibr = indexRect(index);
+    else if (w > h)
+          ibr = indexRect(index) - QMargins(sub, 0, sub, 0);
+    else
+          ibr = indexRect(index) - QMargins(0, 0, 0, sub * 2);
+
+    const double result = x1 * ibr.width() + x2 * iconSize.width();
+    int margin = result > 0 ? result * 0.71 : 1;
+    QRect br = ibr.marginsRemoved(QMargins(margin, 1, margin, margin * 2));
+
+    int iconTopMargin = 6;
+
+    const int iconLeftMargins = (br.width() - iconSize.width()) / 2;
+    return QRect(br.topLeft() + QPoint(iconLeftMargins, iconTopMargin - 2), iconSize);
+}
+
+/**
+ * @brief AppGridView::startDrag 处理listview中app的拖动操作
+ * @param index 被拖动app的对应的模型索引
+ * @param execDrag 如果实际执行了drag操作设置为true，否则传入false
+ */
+void AppGridView::startDrag(const QModelIndex &index, bool execDrag)
 {
     if (!index.isValid())
         return;
+
     m_moveGridView = false;
     AppsListModel *listModel = qobject_cast<AppsListModel *>(model());
     if (!listModel)
@@ -421,52 +488,83 @@ void AppGridView::startDrag(const QModelIndex &index)
     const qreal ratio = qApp->devicePixelRatio();
     QString appKey = index.data(AppsListModel::AppKeyRole).value<QString>();
 
-//    if(appKey == "dde-trash")
-//        return;
-
     QPixmap srcPix = creatSrcPix(index, appKey);
 
     srcPix = srcPix.scaled(m_calcUtil->appIconSize() * ratio, Qt::KeepAspectRatio, Qt::SmoothTransformation);
     srcPix.setDevicePixelRatio(ratio);
 
-    QDrag *drag = new QDrag(this);
-    drag->setMimeData(model()->mimeData(QModelIndexList() << dragIndex));
-    drag->setPixmap(srcPix);
-    drag->setHotSpot(srcPix.rect().center() / ratio);
+    // 创建拖拽释放时的应用图标
+    m_pixLabel->setPixmap(srcPix);
+    m_pixLabel->setFixedSize(srcPix.size() / ratio);
+    m_pixLabel->move(srcPix.rect().center() / ratio);
+
+    QPropertyAnimation *posAni = new QPropertyAnimation(m_pixLabel.data(), "pos", m_pixLabel.data());
+    connect(posAni, &QPropertyAnimation::finished, [&, listModel] () {
+        m_pixLabel->hide();
+        if (!m_lastFakeAni) {
+            if (m_enableDropInside)
+                listModel->dropSwap(m_dropToPos);
+            else
+                // 搜索模式下，不需要做删除被拖拽的item，插入移动到新位置的item的操作，否则会导致被拖拽item的位置改变
+                if (listModel->category() != AppsListModel::AppCategory::Search){
+                    listModel->dropSwap(indexAt(m_dragStartPos).row());
+                }
+
+            listModel->clearDraggingIndex();
+        } else {
+            connect(m_lastFakeAni, &QPropertyAnimation::finished, listModel, &AppsListModel::clearDraggingIndex);
+        }
+
+        setDropAndLastPos(QPoint(0, 0));
+        m_enableDropInside = false;
+        m_lastFakeAni = nullptr;
+    });
 
-    // request remove current item.
     m_dropToPos = index.row();
     listModel->setDraggingIndex(index);
 
-    setState(DraggingState);
-    drag->exec(Qt::MoveAction);
+    int old_page = m_containerBox->property("curPage").toInt();
 
-    // disable animation when finally dropped
-    m_dropThresholdTimer->stop();
-
-    // send to next page
-    emit dragEnd();
-
-
-    if (!m_lastFakeAni) {
-        if (m_enableDropInside)
-            listModel->dropSwap(m_dropToPos);
-        else
-            listModel->dropSwap(indexAt(m_dragStartPos).row());
-
-        listModel->clearDraggingIndex();
-    } else {
-        connect(m_lastFakeAni, &QPropertyAnimation::finished, listModel, &AppsListModel::clearDraggingIndex);
+    if (execDrag) {
+        QDrag *drag = new QDrag(this);
+        drag->setMimeData(model()->mimeData(QModelIndexList() << dragIndex));
+        drag->setPixmap(srcPix);
+        drag->setHotSpot(srcPix.rect().center() / ratio);
+        setState(DraggingState);
+        drag->exec(Qt::MoveAction);
     }
 
-    m_enableDropInside = false;
-}
+    // 拖拽操作完成后暂停app移动动画
+    m_dropThresholdTimer->stop();
 
-bool AppGridView::eventFilter(QObject *o, QEvent *e)
-{
-    Q_UNUSED(o);
-    Q_UNUSED(e);
-    return false;
+    // 未触发分页则直接返回,触发分页则执行分页后操作
+    emit dragEnd();
+
+    int cur_page = m_containerBox->property("curPage").toInt();
+
+    // 当拖动应用出现触发分页的情况，关闭上一个动画， 直接处理当前当前页的动画
+    if (cur_page != old_page) {
+        posAni->stop();
+        return;
+    }
+
+    QRect rectIcon(QPoint(), m_pixLabel->size());
+
+    m_pixLabel->move((QCursor::pos() - rectIcon.center()));
+    m_pixLabel->show();
+
+    posAni->setEasingCurve(QEasingCurve::Linear);
+    posAni->setDuration(DLauncher::APP_DRAG_MININUM_TIME);
+    posAni->setStartValue((QCursor::pos() - rectIcon.center()));
+    posAni->setEndValue(mapToGlobal(m_dropPoint) + QPoint(m_calcUtil->appMarginLeft(), 0));
+
+    //不开启特效则不展示动画
+    if (!DGuiApplicationHelper::isSpecialEffectsEnvironment()) {
+        posAni->setStartValue(posAni->endValue());
+        posAni->setDuration(0);
+    }
+
+    posAni->start(QPropertyAnimation::DeleteWhenStopped);
 }
 
 /**
@@ -476,8 +574,8 @@ bool AppGridView::eventFilter(QObject *o, QEvent *e)
 void AppGridView::fitToContent()
 {
     const QSize size { contentsRect().width(), contentsSize().height() };
-
-    if (size == rect().size()) return;
+    if (size == rect().size())
+        return;
 
     setFixedSize(size);
 }
@@ -520,10 +618,14 @@ void AppGridView::prepareDropSwap()
     if (start == end)
         return;
 
-    for (int i(start + moveToNext); i != end - !moveToNext; ++i)
-        createFakeAnimation(i, moveToNext);
-    // last animation
-    createFakeAnimation(end - !moveToNext, moveToNext, true);
+    int index = 0;
+    for (int i = (start + moveToNext); i != (end - !moveToNext); ++i)
+        createFakeAnimation(i, moveToNext, index++);
+
+    createFakeAnimation(end - !moveToNext, moveToNext, index++, true);
+
+    // item最后回归的位置
+    setDropAndLastPos(appIconRect(dropIndex).topLeft());
 
     m_dragStartPos = indexRect(dropIndex).center();
 }
@@ -534,11 +636,15 @@ void AppGridView::prepareDropSwap()
  * @param moveNext item是否移动的标识
  * @param isLastAni 是最后的那个item移动的动画标识
  */
-void AppGridView::createFakeAnimation(const int pos, const bool moveNext, const bool isLastAni)
+void AppGridView::createFakeAnimation(const int pos, const bool moveNext, const int rIndex, const bool isLastAni)
 {
+    if (rIndex >= m_floatLabels.size())
+        return;
+
+    // listview n行1列,肉眼所及的都是app自动换行后的效果
     const QModelIndex index(indexAt(pos));
 
-    QLabel *floatLabel = new QLabel(this);
+    QLabel *floatLabel = m_floatLabels[rIndex];
     QPropertyAnimation *ani = new QPropertyAnimation(floatLabel, "pos", floatLabel);
 
     const auto ratio = devicePixelRatioF();
@@ -559,12 +665,21 @@ void AppGridView::createFakeAnimation(const int pos, const bool moveNext, const
     floatLabel->setPixmap(pixmap);
     floatLabel->show();
 
-    ani->setStartValue(indexRect(index).topLeft() - QPoint(0, -6));
-    ani->setEndValue(indexRect(indexAt(moveNext ? pos - 1 : pos + 1)).topLeft() - QPoint(0, -6));
-    ani->setEasingCurve(QEasingCurve::Linear);
-    ani->setDuration(300);
+    int topMargin = m_calcUtil->appMarginTop();
+    int leftMargin = m_calcUtil->appMarginLeft();
+    ani->setStartValue(indexRect(index).topLeft() - QPoint(0, -topMargin) + QPoint(leftMargin, 0));
+    ani->setEndValue(indexRect(indexAt(moveNext ? pos - 1 : pos + 1)).topLeft() - QPoint(0, -topMargin) + QPoint(leftMargin, 0));
 
-    connect(ani, &QPropertyAnimation::finished, floatLabel, &QLabel::deleteLater);
+    // InOutQuad 描述起点矩形到终点矩形的速度曲线
+    ani->setEasingCurve(QEasingCurve::Linear);
+    ani->setDuration(DLauncher::APP_DRAG_MININUM_TIME);
+
+    if (!DGuiApplicationHelper::isSpecialEffectsEnvironment()) {
+        ani->setStartValue(ani->endValue());
+        ani->setDuration(0);
+    }
+
+    connect(ani, &QPropertyAnimation::finished, floatLabel, &QLabel::hide);
     if (isLastAni) {
         m_lastFakeAni = ani;
         connect(ani, &QPropertyAnimation::finished, this, &AppGridView::dropSwap);
@@ -594,3 +709,58 @@ const QRect AppGridView::indexRect(const QModelIndex &index) const
 {
     return rectForIndex(index);
 }
+
+/** 提前创建好拖拽过程中需要用到的label
+ * （修复在龙芯下运行时创建对象会崩溃的问题）
+ * @brief AppGridView::createMovingLabel
+ */
+void AppGridView::createMovingComponent()
+{
+    // 拖拽释放鼠标时显示的应用图标
+    if (!m_pixLabel) {
+        m_pixLabel.reset(new QLabel(fullscreen()));
+        m_pixLabel->hide();
+    }
+
+    // 拖拽过程中位置交换时显示的应用图标
+    if (!m_floatLabels.size()) {
+        // 单页最多28个应用
+        for (int i = 0; i < m_calcUtil->appPageItemCount(AppsListModel::All); i++) {
+            QLabel *moveLabel = new QLabel(this);
+            moveLabel->hide();
+            m_floatLabels << moveLabel;
+        }
+    }
+
+    if (!m_calendarWidget) {
+        m_calendarWidget = new QWidget(this);
+        m_calendarWidget->hide();
+    }
+
+    if (!m_vlayout) {
+        m_vlayout = new QVBoxLayout(this);
+    }
+
+    if (!m_monthLabel) {
+        m_monthLabel = new QLabel(this);
+        m_monthLabel->hide();
+    }
+
+    if (!m_dayLabel) {
+        m_dayLabel = new QLabel(this);
+        m_dayLabel->hide();
+    }
+
+    if (!m_weekLabel) {
+        m_weekLabel = new QLabel(this);
+        m_weekLabel->hide();
+    }
+}
+
+FullScreenFrame *AppGridView::fullscreen()
+{
+    FullScreenFrame *fullscreenFrame = nullptr;
+    fullscreenFrame = qobject_cast<FullScreenFrame*>(topLevelWidget());
+
+    return fullscreenFrame;
+}
