@@ -24,8 +24,11 @@
 #include "../fullscreenframe.h"
 #include "../widgets/blurboxwidget.h"
 
+#include <DGuiApplicationHelper>
+
 #include <QHBoxLayout>
-#define     SIDES_SPACE_SCALE   0.10
+
+DGUI_USE_NAMESPACE
 
 /**
  * @brief MultiPagesView::MultiPagesView 全屏模式下多页列表控件类
@@ -36,23 +39,22 @@ MultiPagesView::MultiPagesView(AppsListModel::AppCategory categoryModel, QWidget
     : QWidget(parent)
     , m_pLeftGradient(new GradientLabel(this))
     , m_pRightGradient(new GradientLabel(this))
-    , m_pageCount(0)
-    , m_pageIndex(0)
-    , m_category(categoryModel)
     , m_appsManager(AppsManager::instance())
     , m_calcUtil(CalculateUtil::instance())
     , m_appListArea(new AppListArea)
     , m_viewBox(new DHBoxWidget)
-    , m_pageControl(new pageControl)
+    , m_delegate(Q_NULLPTR)
+    , m_pageControl(new PageControl)
+    , m_pageCount(0)
+    , m_pageIndex(0)
+    , m_category(categoryModel)
+    , m_bDragStart(false)
+    , m_bMousePress(false)
+    , m_nMousePos(0)
+    , m_scrollValue(0)
+    , m_scrollStart(0)
+    , m_changePageDelayTime(nullptr)
 {
-    m_bDragStart = false;
-
-    m_bMousePress = false;
-    m_nMousePos = 0;
-    m_scrollValue = 0;
-    m_scrollStart = 0;
-
-    // 测试用
     m_pRightGradient->setAccessibleName("thisRightGradient");
     m_pLeftGradient->setAccessibleName("thisLeftGradient");
     m_pageControl->setAccessibleName("pageControl");
@@ -69,17 +71,42 @@ MultiPagesView::MultiPagesView(AppsListModel::AppCategory categoryModel, QWidget
     m_appListArea->viewport()->installEventFilter(this);
     m_appListArea->installEventFilter(this);
 
+    if (m_calcUtil->displayMode() == ALL_APPS) {
+        // 全屏视图管理类设置左右边距
+        int padding = m_calcUtil->getScreenSize().width() * DLauncher::SIDES_SPACE_SCALE / 2;
+        m_viewBox->layout()->setContentsMargins(padding, 0, padding, 0);
+        m_viewBox->layout()->setSpacing(padding);
+    }
+
     // 翻页按钮和动画
-    m_pageSwitchAnimation = new QPropertyAnimation(m_appListArea->horizontalScrollBar(), "value");
-    m_pageSwitchAnimation->setEasingCurve(QEasingCurve::OutQuad);
+    m_pageSwitchAnimation = new QPropertyAnimation(m_appListArea->horizontalScrollBar(), "value", this);
+    m_pageSwitchAnimation->setEasingCurve(QEasingCurve::Linear);
+    if (!DGuiApplicationHelper::isSpecialEffectsEnvironment()) {
+        m_changePageDelayTime = new QTime();
+        m_pageSwitchAnimation->setDuration(0);
+    }
 
     InitUI();
 
     connect(m_appListArea, &AppListArea::increaseIcon, this, [ = ] { if (m_calcUtil->increaseIconSize()) emit m_appsManager->layoutChanged(AppsListModel::All); });
     connect(m_appListArea, &AppListArea::decreaseIcon, this, [ = ] { if (m_calcUtil->decreaseIconSize()) emit m_appsManager->layoutChanged(AppsListModel::All); });
-    connect(m_pageControl, &pageControl::onPageChanged, this, &MultiPagesView::showCurrentPage);
+    connect(m_pageControl, &PageControl::onPageChanged, this, &MultiPagesView::showCurrentPage);
 }
 
+MultiPagesView::~MultiPagesView()
+{
+    if (m_changePageDelayTime) {
+        delete m_changePageDelayTime;
+        m_changePageDelayTime = nullptr;
+    }
+}
+
+/**
+ * @brief MultiPagesView::updateGradient 切换分页时屏幕左右两边30pixel的过渡效果
+ * @param pixmap 屏幕背景图片对象
+ * @param topLeftImg 左侧过渡范围起点
+ * @param topRightImg 右侧过渡范围起点
+ */
 void MultiPagesView::updateGradient(QPixmap &pixmap, QPoint topLeftImg, QPoint topRightImg)
 {
     m_pLeftGradient->setDirection(GradientLabel::LeftToRight);
@@ -121,6 +148,7 @@ void MultiPagesView::updateGradient(QPixmap &pixmap, QPoint topLeftImg, QPoint t
 void MultiPagesView::updatePageCount(AppsListModel::AppCategory category)
 {
     int pageCount = m_appsManager->getPageCount(category == AppsListModel::All ? m_category : category);
+
     if (pageCount < 1)
         pageCount = 1;
 
@@ -133,7 +161,7 @@ void MultiPagesView::updatePageCount(AppsListModel::AppCategory category)
             pModel->setPageIndex(m_pageCount);
             m_pageAppsModelList.push_back(pModel);
 
-            AppGridView *pageView = new AppGridView;
+            AppGridView *pageView = new AppGridView(this);
             pageView->setModel(pModel);
             pageView->setItemDelegate(m_delegate);
             pageView->setContainerBox(m_appListArea);
@@ -143,7 +171,7 @@ void MultiPagesView::updatePageCount(AppsListModel::AppCategory category)
 
             m_viewBox->layout()->insertWidget(m_pageCount, pageView);
 
-            m_pageCount ++;
+            m_pageCount++;
 
             connect(pageView, &AppGridView::requestScrollLeft, this, &MultiPagesView::dragToLeft);
             connect(pageView, &AppGridView::requestScrollRight, this, &MultiPagesView::dragToRight);
@@ -153,21 +181,24 @@ void MultiPagesView::updatePageCount(AppsListModel::AppCategory category)
             });
             connect(pageView, &AppGridView::dragEnd, this, &MultiPagesView::dragStop);
             connect(m_pageSwitchAnimation, &QPropertyAnimation::finished,pageView,&AppGridView::setDragAnimationEnable);
-            connect(m_pageSwitchAnimation, &QPropertyAnimation::finished,this, [ = ]{
+            connect(m_pageSwitchAnimation, &QPropertyAnimation::finished,this, [ = ] {
                     setGradientVisible(false);
             });
             emit connectViewEvent(pageView);
+            // 新增的页面需要设置一下大小
+            updatePosition();
         }
     } else {
         while (pageCount < m_pageCount) {
             AppGridView *pageView = qobject_cast<AppGridView *>(m_viewBox->layout()->itemAt(m_pageCount - 1)->widget());
             m_viewBox->layout()->removeWidget(pageView);
+            pageView->model()->deleteLater();
             pageView->deleteLater();
 
             m_pageAppsModelList.removeLast();
             m_appGridViewList.removeLast();
 
-            m_pageCount --;
+            m_pageCount--;
         }
     }
 
@@ -184,7 +215,7 @@ void MultiPagesView::dragToLeft(const QModelIndex &index)
     if (m_pageIndex <= 0)
         return;
 
-    if (m_pageSwitchAnimation->state() == QPropertyAnimation::Running || m_bDragStart)
+    if (isScrolling() || m_bDragStart)
         return;
 
     m_appGridViewList[m_pageIndex]->dragOut(-1);
@@ -193,7 +224,11 @@ void MultiPagesView::dragToLeft(const QModelIndex &index)
 
     int lastApp = m_pageAppsModelList[m_pageIndex]->rowCount(QModelIndex());
     QModelIndex firstModel = m_appGridViewList[m_pageIndex]->indexAt(lastApp - 1);
-    m_appGridViewList[m_pageIndex]->dragIn(firstModel);
+    m_appGridViewList[m_pageIndex]->dragIn(firstModel, m_pageSwitchAnimation->state() != QPropertyAnimation::Running);
+
+    // 保存向左拖拽后item回归的终点位置
+    const QPoint &dropCursorPoint = m_appGridViewList[m_pageIndex]->appIconRect(firstModel).topLeft();
+    m_appGridViewList[m_pageIndex]->setDropAndLastPos(dropCursorPoint);
 
     m_bDragStart = true;
 }
@@ -208,7 +243,7 @@ void MultiPagesView::dragToRight(const QModelIndex &index)
     if (m_pageIndex >= m_pageCount - 1)
         return;
 
-    if (m_pageSwitchAnimation->state() == QPropertyAnimation::Running || m_bDragStart)
+    if (isScrolling() || m_bDragStart)
         return;
 
     // 当前页面准备空出最后一个图标
@@ -222,7 +257,11 @@ void MultiPagesView::dragToRight(const QModelIndex &index)
     // 将最后一个App'挤走'
     int lastApp = m_pageAppsModelList[m_pageIndex]->rowCount(QModelIndex());
     QModelIndex lastModel = m_appGridViewList[m_pageIndex]->indexAt(lastApp - 1);
-    m_appGridViewList[m_pageIndex]->dragIn(lastModel);
+    m_appGridViewList[m_pageIndex]->dragIn(lastModel, m_pageSwitchAnimation->state() != QPropertyAnimation::Running);
+
+    // 保存向右拖拽后item回归的终点位置
+    const QPoint &dropCursorPoint = m_appGridViewList[m_pageIndex]->appIconRect(lastModel).topLeft();
+    m_appGridViewList[m_pageIndex]->setDropAndLastPos(dropCursorPoint);
 
     m_bDragStart = true;
 }
@@ -285,14 +324,45 @@ void MultiPagesView::setModel(AppsListModel::AppCategory category)
     }
 }
 
-void MultiPagesView::updatePosition()
+/**
+ * @brief MultiPagesView::updatePosition
+ * 给自由模式,分类模式下分别给滑动区域, 透明控件, 视图列表设置大小,
+ * 并设置边距,默认回到首页
+ * @param mode 模式分类: 自由模式: 0, 分类模式: 1, 搜索模式: 2
+ */
+void MultiPagesView::updatePosition(int mode)
 {
-    m_pageControl->UpdateIconSize(m_calcUtil->getScreenScaleX(), m_calcUtil->getScreenScaleY());
-    QSize tmpSize = size() - QSize(0,m_pageControl->height() + DLauncher::DRAG_THRESHOLD);
-    m_appListArea->setFixedSize(tmpSize);
-    m_viewBox->setFixedHeight(tmpSize.height());
-    for (auto *pView : m_appGridViewList)
-        pView->setFixedSize(tmpSize);
+    // 更新全屏两种模式下界面布局的左右边距和间隔
+    if (m_calcUtil->displayMode() == ALL_APPS || mode == SEARCH) {
+        int padding = m_calcUtil->getScreenSize().width() * DLauncher::SIDES_SPACE_SCALE / 2;
+        m_viewBox->layout()->setContentsMargins(padding, 0, padding, 0);
+        m_viewBox->layout()->setSpacing(padding);
+    } else {
+        m_viewBox->layout()->setContentsMargins(0, 0, 0, 0);
+        m_viewBox->layout()->setSpacing(0);
+    }
+
+    // 更新视图列表的大小
+    if (m_calcUtil->displayMode() == ALL_APPS || mode == SEARCH) {
+        int padding = m_calcUtil->getScreenSize().width() * DLauncher::SIDES_SPACE_SCALE;
+        m_pageControl->updateIconSize(m_calcUtil->getScreenScaleX(), m_calcUtil->getScreenScaleY());
+
+        QSize tmpSize = size() - QSize(padding, m_pageControl->height() + DLauncher::DRAG_THRESHOLD);
+        m_appListArea->setFixedSize(size());
+        m_viewBox->setFixedHeight(tmpSize.height());
+
+        for (auto pView : m_appGridViewList)
+            pView->setFixedSize(tmpSize);
+    } else {
+        m_pageControl->updateIconSize(m_calcUtil->getScreenScaleX(), m_calcUtil->getScreenScaleY());
+
+        QSize tmpSize = size() - QSize(0, m_pageControl->height() + DLauncher::DRAG_THRESHOLD);
+        m_appListArea->setFixedSize(tmpSize);
+        m_viewBox->setFixedHeight(tmpSize.height());
+
+        for (auto pView : m_appGridViewList)
+            pView->setFixedSize(tmpSize);
+    }
 
     showCurrentPage(0);
 }
@@ -307,22 +377,31 @@ void MultiPagesView::InitUI()
     QVBoxLayout *layoutMain = new QVBoxLayout;
     layoutMain->setContentsMargins(0, 0, 0, DLauncher::DRAG_THRESHOLD);
     layoutMain->setSpacing(0);
-    layoutMain->addWidget(m_appListArea,0,Qt::AlignHCenter);
-    layoutMain->addWidget(m_pageControl,0,Qt::AlignHCenter);
+    layoutMain->addWidget(m_appListArea, 0, Qt::AlignHCenter);
+    layoutMain->addWidget(m_pageControl, 0, Qt::AlignHCenter);
 
     setLayout(layoutMain);
 }
 
 void MultiPagesView::showCurrentPage(int currentPage)
 {
+    int padding = 0;
+    if (m_calcUtil->displayMode() == ALL_APPS)
+        padding = m_calcUtil->getScreenSize().width() * DLauncher::SIDES_SPACE_SCALE / 2;
+
     m_pageIndex = currentPage > 0 ? (currentPage < m_pageCount ? currentPage : m_pageCount - 1) : 0;
-    int endValue = m_pageIndex == 0 ? 0 : m_appGridViewList[m_pageIndex]->x();
+    int endValue = m_pageIndex == 0 ? 0 : (m_appGridViewList[m_pageIndex]->x() - padding);
     int startValue = m_appListArea->horizontalScrollBar()->value();
+    m_appListArea->setProperty("curPage", m_pageIndex);
+
     m_pageSwitchAnimation->stop();
     m_pageSwitchAnimation->setStartValue(startValue);
     m_pageSwitchAnimation->setEndValue(endValue);
     m_pageSwitchAnimation->start();
 
+    if (m_changePageDelayTime)
+        m_changePageDelayTime->start();
+
     m_pageControl->setCurrent(m_pageIndex);
 }
 
@@ -371,16 +450,16 @@ AppsListModel *MultiPagesView::pageModel(int pageIndex)
 
 void MultiPagesView::wheelEvent(QWheelEvent *e)
 {
-    if (m_pageSwitchAnimation->state() == QPropertyAnimation::Running)
+    if (isScrolling())
         return;
 
     int page = m_pageIndex;
     if (e->delta() > 0) {
         if (page - 1 >= 0)
-            -- page;
+            --page;
     } else if (e->delta() < 0) {
         if (page + 1 < m_pageCount)
-            ++ page;
+            ++page;
     }
 
     if (page != m_pageIndex)
@@ -394,8 +473,6 @@ void MultiPagesView::mousePress(QMouseEvent *e)
     m_scrollValue = m_appListArea->horizontalScrollBar()->value();
     m_scrollStart = m_scrollValue;
 
-    if(e->button() != Qt::RightButton) updateGradient();
-
     if(m_pageCount == 1 && m_category != AppsListModel::Search)
         QWidget::mousePressEvent(e);
 }
@@ -404,8 +481,10 @@ void MultiPagesView::mouseMove(QMouseEvent *e)
 {
     if (!m_bMousePress)
         return;
+
     int nDiff = m_nMousePos - e->x();
     m_scrollValue += nDiff;
+
     m_appListArea->horizontalScrollBar()->setValue(m_scrollValue);
 
     if(m_pageCount == 1)
@@ -415,13 +494,14 @@ void MultiPagesView::mouseMove(QMouseEvent *e)
 void MultiPagesView::mouseRelease(QMouseEvent *e)
 {
     int nDiff = m_nMousePos - e->x();
+
     if (nDiff > DLauncher::TOUCH_DIFF_THRESH) { // 加大范围来避免手指点击触摸屏抖动问题
         showCurrentPage(m_pageIndex + 1);
     } else if (nDiff < -DLauncher::TOUCH_DIFF_THRESH) { // 加大范围来避免手指点击触摸屏抖动问题
         showCurrentPage(m_pageIndex - 1);
     } else {
         int nScroll = m_appListArea->horizontalScrollBar()->value();
-        //多个分页是点击直接隐藏
+        // 多个分页是点击直接隐藏
         if (nScroll == m_scrollStart && m_pageCount != 1)
             emit m_appGridViewList[m_pageIndex]->clicked(QModelIndex());
         else if (nScroll - m_scrollStart > DLauncher::MOUSE_MOVE_TO_NEXT)
@@ -450,7 +530,7 @@ QPropertyAnimation::State MultiPagesView::getPageSwitchAnimationState()
     return m_pageSwitchAnimation->state();
 }
 
-QWidget* MultiPagesView::getParentWidget()
+QWidget *MultiPagesView::getParentWidget()
 {
     // 找到背景widget
     QWidget *backgroundWidget = parentWidget();
@@ -463,6 +543,7 @@ QWidget* MultiPagesView::getParentWidget()
 
         backgroundWidget = backgroundWidget->parentWidget();
     }
+
     return backgroundWidget;
 }
 
@@ -470,17 +551,34 @@ QPoint MultiPagesView::calculPadding(MultiPagesView::Direction dir)
 {
     // 获取当前屏幕的高度
     QScreen *screen = QGuiApplication::primaryScreen();
-    int screen_height = screen->availableGeometry().height();
+    int screenHeight = screen->availableGeometry().height();
+
     // 计算过渡动画开始的位置 区分左右位置与上下位置
-    int paddingL = m_calcUtil->getScreenSize().width() * SIDES_SPACE_SCALE;
+    int paddingL = m_calcUtil->getScreenSize().width() * DLauncher::SIDES_SPACE_SCALE;
     int paddingR = m_calcUtil->getScreenSize().width() - paddingL - 1;
-    int topPos = mapToGlobal(rect().topLeft()).y() > screen_height ? mapToGlobal(rect().topLeft()).y() - screen_height : mapToGlobal(rect().topLeft()).y();
+    int topPos = (mapToGlobal(rect().topLeft()).y() > screenHeight) ?
+                 (mapToGlobal(rect().topLeft()).y() - screenHeight) : mapToGlobal(rect().topLeft()).y();
 
     QPoint padding(((dir == MultiPagesView::Left) ? paddingL : paddingR), topPos);
 
     return padding;
 }
 
+AppListArea *MultiPagesView::getListArea()
+{
+    return m_appListArea;
+}
+
+AppGridViewList MultiPagesView::getAppGridViewList()
+{
+    return m_appGridViewList;
+}
+
+AppsListModel::AppCategory MultiPagesView::getCategory()
+{
+    return m_category;
+}
+
 // 更新边框渐变，在屏幕变化时需要更新，类别拖动时需要隐藏
 void MultiPagesView::updateGradient()
 {
@@ -491,3 +589,11 @@ void MultiPagesView::updateGradient()
     QPixmap background = backgroundWidget->grab();
     updateGradient(background, calculPadding(MultiPagesView::Left), calculPadding(MultiPagesView::Right));
 }
+
+bool MultiPagesView::isScrolling()
+{
+    if (m_changePageDelayTime)
+        return m_changePageDelayTime->isValid() && m_changePageDelayTime->elapsed() < DLauncher::CHANGE_PAGE_DELAY_TIME;
+
+    return m_pageSwitchAnimation->state() == QPropertyAnimation::Running;
+}
