@@ -51,7 +51,7 @@ DWIDGET_USE_NAMESPACE
 
 using Appearance = com::deepin::daemon::Appearance;
 class QLabel;
-
+class DBusDockInterface;
 class WindowedFrame : public DBlurEffectWidget, public LauncherInterface
 {
     Q_OBJECT
@@ -129,7 +129,7 @@ private slots:
     void prepareHideLauncher();
     void recoveryAll();
     void onOpacityChanged(const double value);
-    void onScreenInfoChange();
+    void onDockGeometryChanged();
     void updatePosition();
     void onHideMenu();
 
@@ -144,7 +144,6 @@ private:
     AppsManager *m_appsManager;
     AppListView *m_appsView;
     AppsListModel *m_appsModel;          // 应用列表
-    AppsListModel *m_usedModel;
     AppsListModel *m_searchModel;        // 搜索后的应用列表
 
     QWidget *m_rightWidget;
@@ -165,7 +164,8 @@ private:
     ModeToggleButton *m_modeToggleBtn;   // 小窗口右上角窗口模式切换按钮
     DSearchEdit *m_searcherEdit;         // 搜索控件
     bool m_enterSearchEdit;
-    const QScreen *m_curScreen;
+    const DBusDockInterface *m_dockFrontInfc = nullptr;
+    int m_currentAppListIndex;
 };
 
 #endif // WINDOWEDFRAME_H
